@@ -17,8 +17,13 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 #
 
-__version__ = '2.10.0'
+__version__ = '2.11.0'
 
+from pytz import utc as LOCAL_TZ
+
+from spyne._base import BODY_STYLE_WRAPPED
+from spyne._base import BODY_STYLE_BARE
+from spyne._base import BODY_STYLE_EMPTY
 from spyne._base import AuxMethodContext
 from spyne._base import TransportContext
 from spyne._base import EventContext
@@ -26,9 +31,15 @@ from spyne._base import MethodContext
 from spyne._base import MethodDescriptor
 from spyne._base import EventManager
 
-import sys
+from spyne.decorator import rpc
+from spyne.decorator import srpc
+from spyne.decorator import mrpc
+from spyne.service import ServiceBase
+from spyne.application import Application
+from spyne.model import *
 
-if not hasattr(sys, "version_info") or sys.version_info < (2, 5):
-    raise RuntimeError("Spyne requires Python 2.5 or later.")
-
-del sys
+def _vercheck():
+    import sys
+    if not hasattr(sys, "version_info") or sys.version_info < (2, 6):
+        raise RuntimeError("Spyne requires Python 2.6 or later. Trust us.")
+_vercheck()
