@@ -21,7 +21,7 @@ import logging
 logger = logging.getLogger(__name__)
 
 from time import time
-
+from copy import copy
 from collections import deque
 
 from spyne.const.xml_ns import DEFAULT_NS
@@ -34,8 +34,8 @@ class BODY_STYLE_BARE: pass
 
 class AuxMethodContext(object):
     """Generic object that holds information specific to auxiliary methods"""
-    def __init__(self, p_ctx, error):
-        self.p_ctx = p_ctx
+    def __init__(self, parent, error):
+        self.parent = parent
         """Primary context that this method was bound to."""
 
         self.error = error
@@ -44,7 +44,10 @@ class AuxMethodContext(object):
 
 class TransportContext(object):
     """Generic object that holds transport-specific context information"""
-    def __init__(self, transport, type=None):
+    def __init__(self, parent, transport, type=None):
+        self.parent = parent;
+        """The MethodContext this object belongs to"""
+
         self.itself = transport
         """The transport itself; i.e. a ServerBase instance."""
 
@@ -53,10 +56,17 @@ class TransportContext(object):
 
         self.app = transport.app
 
+        self.request_encoding = None
+        """General purpose variable to hold the string identifier of a request
+        encoding. It's nowadays usually 'utf-8', especially with http data"""
+
 
 class ProtocolContext(object):
     """Generic object that holds transport-specific context information"""
-    def __init__(self, transport, type=None):
+    def __init__(self, parent, transport, type=None):
+        self.parent = parent;
+        """The MethodContext this object belongs to"""
+
         self.itself = transport
         """The transport itself; i.e. a ServerBase instance."""
 
@@ -66,8 +76,9 @@ class ProtocolContext(object):
 
 class EventContext(object):
     """Generic object that holds event-specific context information"""
-    def __init__(self, event_id=None):
-        self.event_id=event_id
+    def __init__(self, parent, event_id=None):
+        self.parent = parent
+        self.event_id = event_id
 
 
 class MethodContext(object):
@@ -77,6 +88,20 @@ class MethodContext(object):
 
     frozen = False
 
+    def copy(self):
+        retval = copy(self)
+
+        if retval.transport is not None:
+            retval.transport.parent = retval
+        if retval.protocol is not None:
+            retval.protocol.parent = retval
+        if retval.event is not None:
+            retval.event.parent = retval
+        if retval.aux is not None:
+            retval.aux.parent = retval
+
+        return retval
+
     @property
     def method_name(self):
         """The public name of the method the ``method_request_string`` was
@@ -108,15 +133,15 @@ class MethodContext(object):
         self.udc = None
         """The user defined context. Use it to your liking."""
 
-        self.transport = TransportContext(transport)
+        self.transport = TransportContext(self, transport)
         """The transport-specific context. Transport implementors can use this
         to their liking."""
 
-        self.protocol = ProtocolContext(transport)
+        self.protocol = ProtocolContext(self, transport)
         """The protocol-specific context. Protocol implementors can use this
         to their liking."""
 
-        self.event = EventContext()
+        self.event = EventContext(self)
         """Event-specific context. Use this as you want, preferably only in
         events, as you'd probably want to separate the event data from the
         method data."""
@@ -132,10 +157,6 @@ class MethodContext(object):
 
         self.__descriptor = None
 
-        # This is set based on the value of the descriptor.
-        self.service_class = None
-        """The service definition class the method belongs to."""
-
         #
         # Input
         #
@@ -216,7 +237,16 @@ class MethodContext(object):
 
         # stream
         self.out_string = None
-        """Outgoing bytestream (i.e. a sequence of strings)"""
+        """The pull interface to the outgoing bytestream. It's a sequence of
+        strings (which could also be a generator)."""
+
+        self.out_stream = None
+        """The push interface to the outgoing bytestream. It's a file-like
+        object."""
+
+        #self.out_stream = None
+        #"""The push interface to the outgoing bytestream. It's a file-like
+        #object."""
 
         self.function = None
         """The callable of the user code."""
@@ -252,8 +282,15 @@ class MethodContext(object):
     by the user code.
     """
 
+    # Deprecated. Use self.descriptor.service_class.
+    @property
+    def service_class(self):
+        if self.descriptor is not None:
+            return self.descriptor.service_class
+
     def __setattr__(self, k, v):
-        if self.frozen == False or k in self.__dict__ or k == 'descriptor':
+        if not self.frozen or k in self.__dict__ or k in \
+                                                ('descriptor', 'out_protocol'):
             object.__setattr__(self, k, v)
         else:
             raise ValueError("use the udc member for storing arbitrary data "
@@ -281,28 +318,53 @@ class MethodContext(object):
         self.call_end = time()
         self.app.event_manager.fire_event("method_context_closed", self)
 
+    def set_out_protocol(self, what):
+        self._out_protocol = what
+
+    def get_out_protocol(self):
+        return self._out_protocol
+
+    out_protocol = property(get_out_protocol, set_out_protocol)
+
 
 class MethodDescriptor(object):
-    '''This class represents the method signature of an exposed service. It is
+    """This class represents the method signature of an exposed service. It is
     produced by the :func:`spyne.decorator.srpc` decorator.
-    '''
+    """
 
     def __init__(self, function, in_message, out_message, doc,
                  is_callback=False, is_async=False, mtom=False, in_header=None,
                  out_header=None, faults=None,
                  port_type=None, no_ctx=False, udp=None, class_key=None,
-                 aux=None, patterns=None, body_style=None, args=None):
+                 aux=None, patterns=None, body_style=None, args=None,
+                 operation_name=None, no_self=None, translations=None, when=None,
+                 in_message_name_override=True, out_message_name_override=True,
+                 service_class=None, href=None):
 
         self.__real_function = function
         """The original callable for the user code."""
 
         self.reset_function()
 
+        self.operation_name = operation_name
+        """The base name of an operation without the request suffix, as
+        generated by the ``@srpc`` decorator."""
+
         self.in_message = in_message
         """A :class:`spyne.model.complex.ComplexModel` subclass that defines the
         input signature of the user function and that was automatically
         generated by the ``@srpc`` decorator."""
 
+        self.name = None
+        """The public name of the function. Equals to the type_name of the
+        in_message."""
+
+        if body_style is BODY_STYLE_BARE:
+            self.name = in_message.Attributes.sub_name
+
+        if self.name is None:
+            self.name = self.in_message.get_type_name()
+
         self.out_message = out_message
         """A :class:`spyne.model.complex.ComplexModel` subclass that defines the
         output signature of the user function and that was automatically
@@ -370,11 +432,52 @@ class MethodDescriptor(object):
         self.args = args
         """A sequence of the names of the exposed arguments, or None."""
 
-    @property
-    def name(self):
-        """The public name of the function. Equals to the type_name of the
-        in_message."""
-        return self.in_message.get_type_name()
+        # FIXME: docstring yo.
+        self.no_self = no_self
+        """FIXME: docstring yo."""
+
+        self.service_class = service_class
+        """The ServiceBase subclass the method belongs to, if there's any."""
+
+        self.parent_class = None
+        """The ComplexModel subclass the method belongs to. Only set for @mrpc
+        methods."""
+
+        # HATEOAS Stuff
+        self.translations = translations
+        """None or a dict of locale-translation pairs."""
+
+        self.href = href
+        """None or a dict of locale-translation pairs."""
+
+        self.when = when
+        """None or a callable that takes the object instance and returns a
+        boolean value. If true, the object can process that action.
+        """
+
+        self.in_message_name_override = in_message_name_override
+        """When False, no mangling of in message name will be performed by later
+        stages of the interface generation. Naturally, it will be up to you to
+        resolve name clashes."""
+
+        self.out_message_name_override = out_message_name_override
+        """When False, no mangling of out message name will be performed by
+        later stages of the interface generation. Naturally, it will be up to
+        you to resolve name clashes."""
+
+    def translate(self, locale, default):
+        """
+        :param cls: class
+        :param locale: locale string
+        :param default: default string if no translation found
+        :returns: translated string
+        """
+
+        if locale is None:
+            locale = 'en_US'
+        if self.translations is not None:
+            return self.translations.get(locale, default)
+        return default
 
     @property
     def key(self):
@@ -440,3 +543,16 @@ class EventManager(object):
         handlers = self.handlers.get(event_name, oset())
         for handler in handlers:
             handler(ctx)
+
+
+class FakeContext(object):
+    def __init__(self, app=None, descriptor=None, out_object=None, out_error=None,
+                                             out_document=None, out_string=None):
+        self.app = app
+        self.descriptor = descriptor
+        self.out_error = out_error
+        self.out_object = out_object
+        self.out_document = out_document
+        self.out_string = out_string
+        self.protocol = type("ProtocolContext", (object,), {})()
+        self.transport = type("ProtocolContext", (object,), {})()
