@@ -62,7 +62,7 @@ def check_method_port(service, method):
         if (not method.port_type is None):
             index = service.__port_types__.index(method.port_type)
 
-    except ValueError, e:
+    except ValueError as e:
         raise ValueError("""
             The port specified in the rpc decorator does not match any of
             the ports defined by the service class
@@ -146,7 +146,7 @@ class Wsdl11(XmlSchema):
     #    xsd, xsi, wsdl, etc.
 
     def __init__(self, interface=None, _with_partnerlink=False):
-        XmlSchema.__init__(self, interface)
+        super(Wsdl11, self).__init__(interface)
 
         self._with_plink = _with_partnerlink
 
@@ -178,7 +178,7 @@ class Wsdl11(XmlSchema):
         return pt
 
     def _get_or_create_service_node(self, service_name):
-        '''Builds a wsdl:service element.'''
+        """Builds a wsdl:service element."""
 
         ser = None
         if not service_name in self.service_elt_dict:
@@ -200,7 +200,6 @@ class Wsdl11(XmlSchema):
         self.build_schema_nodes()
 
         self.url = REGEX_WSDL.sub('', url)
-        #self.url = url
 
         service_name = self.interface.get_name()
 
@@ -326,8 +325,6 @@ class Wsdl11(XmlSchema):
             binding_name = self._get_binding_name(service_name)
             port_binding_names.append((service_name, binding_name))
 
-        port_name = port_type.get('name')
-
         for method in service.public_methods.values():
             check_method_port(service, method)
 
@@ -338,23 +335,24 @@ class Wsdl11(XmlSchema):
                 operation = etree.SubElement(port_type, '{%s}operation'
                                                                     % _ns_wsdl)
 
-            operation.set('name', method.name)
+            operation.set('name', method.operation_name)
 
             if method.doc is not None:
                 documentation = etree.SubElement(operation, '{%s}documentation'
                                                                     % _ns_wsdl)
                 documentation.text = method.doc
 
-            operation.set('parameterOrder', method.in_message.get_type_name())
+            operation.set('parameterOrder', method.in_message.get_element_name())
 
             op_input = etree.SubElement(operation, '{%s}input' % _ns_wsdl)
-            op_input.set('name', method.in_message.get_type_name())
-            op_input.set('message', method.in_message.get_type_name_ns(self.interface))
+            op_input.set('name', method.in_message.get_element_name())
+            op_input.set('message', method.in_message.get_element_name_ns(self.interface))
 
             if (not method.is_callback) and (not method.is_async):
                 op_output = etree.SubElement(operation, '{%s}output' % _ns_wsdl)
-                op_output.set('name', method.out_message.get_type_name())
-                op_output.set('message', method.out_message.get_type_name_ns(self.interface))
+                op_output.set('name', method.out_message.get_element_name())
+                op_output.set('message', method.out_message.get_element_name_ns(
+                                                                self.interface))
 
                 if not (method.faults is None):
                     for f in method.faults:
@@ -380,17 +378,18 @@ class Wsdl11(XmlSchema):
                 objs = obj
             else:
                 objs = (obj,)
+
             for obj in objs:
                 part = etree.SubElement(message, '{%s}part' % _ns_wsdl)
-                part.set('name', obj.get_type_name())
-                part.set('element', obj.get_type_name_ns(self.interface))
+                part.set('name', obj.get_element_name())
+                part.set('element', obj.get_element_name_ns(self.interface))
 
     def add_messages_for_methods(self, service, root, messages):
         for method in service.public_methods.values():
             self._add_message_for_object(root, messages, method.in_message,
-                                    method.in_message.get_type_name())
+                                    method.in_message.get_element_name())
             self._add_message_for_object(root, messages, method.out_message,
-                                    method.out_message.get_type_name())
+                                    method.out_message.get_element_name())
 
             if method.in_header is not None:
                 if len(method.in_header) > 1:
@@ -421,16 +420,16 @@ class Wsdl11(XmlSchema):
 
         def inner(method, binding):
             operation = etree.Element('{%s}operation' % _ns_wsdl)
-            operation.set('name', method.name)
+            operation.set('name', method.operation_name)
 
             soap_operation = etree.SubElement(operation, '{%s}operation'
                                                                     % _ns_soap)
-            soap_operation.set('soapAction', method.name)
+            soap_operation.set('soapAction', method.operation_name)
             soap_operation.set('style', 'document')
 
             # get input
             input = etree.SubElement(operation, '{%s}input' % _ns_wsdl)
-            input.set('name', method.in_message.get_type_name())
+            input.set('name', method.in_message.get_element_name())
 
             soap_body = etree.SubElement(input, '{%s}body' % _ns_soap)
             soap_body.set('use', 'literal')
@@ -462,7 +461,7 @@ class Wsdl11(XmlSchema):
 
             if not (method.is_async or method.is_callback):
                 output = etree.SubElement(operation, '{%s}output' % _ns_wsdl)
-                output.set('name', method.out_message.get_type_name())
+                output.set('name', method.out_message.get_element_name())
 
                 soap_body = etree.SubElement(output, '{%s}body' % _ns_soap)
                 soap_body.set('use', 'literal')
