@@ -17,81 +17,65 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 #
 
+"""This module contains the ModelBase class and other building blocks for
+defining models.
+"""
+
 import re
 
 import spyne.const.xml_ns
 
 from decimal import Decimal
 
-"""This module contains the ModelBase class and other building blocks for
-defining models.
-"""
+from spyne.util import Break
+from spyne.util.odict import odict
+from spyne.util.six import add_metaclass
 
-
-def nillable_dict(func):
-    """Decorator that retuns empty dictionary if input is None"""
-
-    def wrapper(cls, element):
-        if element is None:
-            return {}
-        else:
-            return func(cls, element)
-    return wrapper
-
-
-def nillable_string(func):
-    """Decorator that retuns None if input is None."""
-
-    def wrapper(cls, string, *args, **kwargs):
-        if string is None:
-            return None
-        else:
-            return func(cls, string, *args, **kwargs)
-    return wrapper
-
-
-def nillable_iterable(func):
-    """Decorator that retuns [] if input is None."""
-
-    def wrapper(prot, cls, string):
-        if string is None:
-            return []
-        else:
-            return func(prot, cls, string)
-    return wrapper
+from spyne.const.xml_ns import DEFAULT_NS
 
 
+# All this code to get rid of a one letter quirk: nillable vs nullable.
 class AttributesMeta(type(object)):
-    """I hate quirks. This is a 10-minute attempt to get rid of a one-letter
-    quirk."""
+    NULLABLE_DEFAULT = True
 
     def __new__(cls, cls_name, cls_bases, cls_dict):
         # Mapper args should not be inherited.
         if not 'sqla_mapper_args' in cls_dict:
             cls_dict['sqla_mapper_args'] = None
 
-        return type(object).__new__(cls, cls_name, cls_bases, cls_dict)
+        return super(AttributesMeta, cls).__new__(cls, cls_name, cls_bases, cls_dict)
 
     def __init__(self, cls_name, cls_bases, cls_dict):
         nullable = cls_dict.get('nullable', None)
         nillable = cls_dict.get('nillable', None)
+        if nullable is not None:
+            assert nillable is None or nullable == nillable
+            self._nullable = nullable
 
-        assert nullable is None or nillable is None or nullable == nillable
+        elif nillable is not None:
+            assert nullable is None or nullable == nillable
+            self._nullable = nillable
 
-        self.__nullable = nullable or nillable or True
+        if getattr(self, '_nullable', None) is None:
+            self._nullable = None
 
-        type(object).__init__(self, cls_name, cls_bases, cls_dict)
+        super(AttributesMeta, self).__init__(cls_name, cls_bases, cls_dict)
 
     def get_nullable(self):
-        return self.__nullable
+        return (self._nullable if self._nullable is not None else
+                self.NULLABLE_DEFAULT)
+
     def set_nullable(self, what):
-        self.__nullable = what
+        self._nullable = what
+
     nullable = property(get_nullable, set_nullable)
 
     def get_nillable(self):
-        return self.__nullable
+        return self.nullable
+
     def set_nillable(self, what):
-        self.__nullable = what
+        self.nullable = what
+
     nillable = property(get_nillable, set_nillable)
 
 
@@ -102,7 +86,8 @@ class ModelBase(object):
     """
 
     __orig__ = None
-    """This holds the original class the class .customize()d from. """
+    """This holds the original class the class .customize()d from. Ie if this is
+    None, the class is not a customize()d one."""
 
     __extends__ = None
     """This holds the original class the class inherited or .customize()d from.
@@ -123,11 +108,10 @@ class ModelBase(object):
     #
     # Please note that min_occurs and max_occurs must be validated in the
     # ComplexModelBase deserializer.
+    @add_metaclass(AttributesMeta)
     class Attributes(object):
         """The class that holds the constraints for the given type."""
 
-        __metaclass__ = AttributesMeta
-
         _wrapper = False
         # when skip_wrappers=True is passed to a protocol, these objects
         # are skipped. just for internal use.
@@ -135,9 +119,13 @@ class ModelBase(object):
         default = None
         """The default value if the input is None"""
 
-        nillable = True
+        default_factory = None
+        """The default value if the input is None"""
+
+        nillable = None
         """Set this to false to reject null values. Synonyms with
-        ``nullable``."""
+        ``nullable``. True by default. The default value can be changed by
+         setting ``AttributesMeta.NULLABLE_DEFAULT``."""
 
         min_occurs = 0
         """Set this to 1 to make this object mandatory. Can be set to any
@@ -158,6 +146,16 @@ class ModelBase(object):
         names to that language as values.
         """
 
+        sub_ns = None
+        """An Xml-specific attribute that specifies which namespace should be
+        used for field names in classes.
+        """
+
+        sub_name = None
+        """This specifies which string should be used as field name when this
+        type is seriazed under a ComplexModel.
+        """
+
         sqla_column_args = None
         """A dict that will be passed to SQLAlchemy's ``Column`` constructor as
         ``**kwargs``.
@@ -193,6 +191,14 @@ class ModelBase(object):
         ``sqlalchemy.Integer``.
         """
 
+        table_name = None
+        """Database table name."""
+
+        xml_choice_group = None
+        """When not None, shares the same <choice> tag with fields with the same
+        xml_choice_group value.
+        """
+
         index = None
         """Can be ``True``, a string, or a tuple of two strings.
 
@@ -207,6 +213,23 @@ class ModelBase(object):
           above.
         """
 
+        read_only= False
+        """If True, the attribute won't be initialized from outside values."""
+
+        prot_attrs = None
+        """to be implemented."""
+        #"""Customize child attributes for protocols. It's a dict of dicts.
+        #The key is either a ProtocolBase subclass or a ProtocolBase instance.
+        #Instances override classes."""
+
+        empty_is_none = False
+        """When the incoming object is empty (e.g. '' for strings) treat it as
+        None. No effect (yet) for outgoing values."""
+
+        order = None
+        """An integer that's passed to ``_type_info.insert()`` as first argument
+         when not None. ``.append()`` is used otherwise."""
+
     class Annotations(object):
         """The class that holds the annotations for the given type."""
 
@@ -249,6 +272,7 @@ class ModelBase(object):
 
         return cls.__namespace__
 
+    # TODO: rename to "resolve_identifier"
     @staticmethod
     def resolve_namespace(cls, default_ns, tags=None):
         """This call finalizes the namespace assignment. The default namespace
@@ -256,6 +280,12 @@ class ModelBase(object):
         of the interface generator.
         """
 
+        if tags is None:
+            tags = set()
+        elif cls in tags:
+            return False
+        tags.add(cls)
+
         if cls.__namespace__ is spyne.const.xml_ns.DEFAULT_NS:
             cls.__namespace__ = default_ns
 
@@ -273,6 +303,17 @@ class ModelBase(object):
 
             cls.__namespace__ = '.'.join(ret)
 
+        if cls.__namespace__ is None or len(cls.__namespace__) == 0:
+            cls.__namespace__ = default_ns
+
+        if cls.__namespace__ is None or len(cls.__namespace__) == 0:
+            raise ValueError("You need to explicitly set %r.__namespace__" % cls)
+
+        if getattr(cls, '__extends__', None) != None:
+            cls.__extends__.resolve_namespace(cls.__extends__, default_ns, tags)
+
+        return True
+
     @classmethod
     def get_type_name(cls):
         """Returns the class name unless the __type_name__ attribute is defined.
@@ -290,24 +331,28 @@ class ModelBase(object):
         """
 
         if cls.get_namespace() != None:
-            return "%s:%s" % (cls.get_namespace_prefix(interface), cls.get_type_name())
+            return "%s:%s" % (cls.get_namespace_prefix(interface),
+                                                            cls.get_type_name())
+
+    @classmethod
+    def get_element_name(cls):
+        return cls.Attributes.sub_name or cls.get_type_name()
+
+    @classmethod
+    def get_element_name_ns(cls, interface):
+        ns = cls.Attributes.sub_ns or cls.get_namespace()
+        if ns is DEFAULT_NS:
+            ns = interface.get_tns()
+        if ns is not None:
+            pref = interface.get_namespace_prefix(ns)
+            return "%s:%s" % (pref, cls.get_element_name())
 
     @classmethod
-    @nillable_string
     def to_string(cls, value):
         """Returns str(value). This should be overridden if this is not enough.
         """
-
         return str(value)
 
-    @classmethod
-    @nillable_dict
-    def to_dict(cls, value):
-        """Returns a dict with type name as key and str(value) as value. This
-        should be overridden if this is not enough."""
-
-        return {cls.get_type_name(): cls.to_string(value)}
-
     @classmethod
     def customize(cls, **kwargs):
         """Duplicates cls and overwrites the values in ``cls.Attributes`` with
@@ -325,17 +370,20 @@ class ModelBase(object):
         Not meant to be overridden.
         """
 
-        cls_dict = {}
+        cls_dict = odict({'__module__': cls.__module__})
+
         if getattr(cls, '__orig__', None) is None:
             cls_dict['__orig__'] = cls
 
         class Attributes(cls.Attributes):
-            translations = {}
-            sqla_column_args = (), {}
+            pass
+
+        if cls.Attributes.translations is None:
+            Attributes.translations = {}
+        if cls.Attributes.sqla_column_args is None:
+            Attributes.sqla_column_args = (), {}
+
         cls_dict['Attributes'] = Attributes
-        # as nillable is a property, it gets reset everytime a new class is
-        # defined.
-        Attributes.nillable = cls.Attributes.nillable
 
         # as nillable is a property, it gets reset everytime a new class is
         # defined. So we need to reinitialize it explicitly.
@@ -352,19 +400,19 @@ class ModelBase(object):
             elif k in ("doc", "appinfo"):
                 setattr(Annotations, k, v)
 
-            elif k in ('primary_key','pk'):
+            elif k in ('primary_key', 'pk'):
                 Attributes.sqla_column_args[-1]['primary_key'] = v
 
-            elif k in ('autoincrement'):
-                Attributes.sqla_column_args[-1][k] = v
-
-            elif k in ('foreign_key','fk'):
+            elif k in ('foreign_key', 'fk'):
                 from sqlalchemy.schema import ForeignKey
                 t, d = Attributes.sqla_column_args
                 fkt = (ForeignKey(v),)
                 Attributes.sqla_column_args = (t + fkt, d)
 
-            elif k == 'max_occurs' and v == 'unbounded':
+            elif k in ('autoincrement', 'onupdate', 'server_default'):
+                Attributes.sqla_column_args[-1][k] = v
+
+            elif k == 'max_occurs' and v in ('unbounded', 'inf', float('inf')):
                 setattr(Attributes, k, Decimal('inf'))
 
             else:
@@ -386,33 +434,39 @@ class ModelBase(object):
         value. This is called after converting the incoming string to the
         native python value."""
 
-        return True
+        return (cls.Attributes.nullable or value is not None)
 
 
 class Null(ModelBase):
     pass
 
 
+class SimpleModelAttributesMeta(AttributesMeta):
+    def __init__(self, cls_name, cls_bases, cls_dict):
+        super(SimpleModelAttributesMeta, self).__init__(cls_name, cls_bases, cls_dict)
+        if getattr(self, '_pattern', None) is None:
+            self._pattern = None
+
+    def get_pattern(self):
+        return self._pattern
+
+    def set_pattern(self, pattern):
+        self._pattern = pattern
+        if pattern is not None:
+            self._pattern_re = re.compile(pattern)
+
+    pattern = property(get_pattern, set_pattern)
+
+
 class SimpleModel(ModelBase):
     """The base class for primitives."""
 
     __namespace__ = "http://www.w3.org/2001/XMLSchema"
 
+    @add_metaclass(SimpleModelAttributesMeta)
     class Attributes(ModelBase.Attributes):
         """The class that holds the constraints for the given type."""
 
-        class __metaclass__(AttributesMeta):
-            def __init__(self, cls_name, cls_bases, cls_dict):
-                AttributesMeta.__init__(self, cls_name, cls_bases, cls_dict)
-                self.__pattern = cls_dict.get('pattern', None)
-            def get_pattern(self):
-                return self.__pattern
-            def set_pattern(self, pattern):
-                self.__pattern = pattern
-                if pattern is not None:
-                    self._pattern_re = re.compile(pattern)
-            pattern = property(get_pattern, set_pattern)
-
         values = set()
         """The set of possible values for this type."""
 
@@ -425,12 +479,23 @@ class SimpleModel(ModelBase):
         See spyne.model.base.ModelBase for more information.
         """
 
-        retval = cls.customize(**kwargs)
+        return cls.customize(**kwargs)
+
+    @classmethod
+    def customize(cls, **kwargs):
+        """Duplicates cls and overwrites the values in ``cls.Attributes`` with
+        ``**kwargs`` and returns the new class."""
+
+        cls_name, cls_bases, cls_dict = cls._s_customize(cls, **kwargs)
+
+        retval = type(cls_name, cls_bases, cls_dict)
 
         if not retval.is_default(retval):
             retval.__extends__ = cls
             retval.__type_name__ = kwargs.get("type_name", ModelBase.Empty)
 
+        retval.resolve_namespace(retval, kwargs.get('__namespace__'))
+
         return retval
 
     @staticmethod
@@ -438,8 +503,138 @@ class SimpleModel(ModelBase):
         return (cls.Attributes.values == SimpleModel.Attributes.values)
 
     @staticmethod
-    def validate_string(cls, value):
-        return (     ModelBase.validate_string(cls, value)
-                and (len(cls.Attributes.values) == 0 or
-                                                value in cls.Attributes.values)
+    def validate_native(cls, value):
+        return (     ModelBase.validate_native(cls, value)
+                and (cls.Attributes.values is None or
+                                            len(cls.Attributes.values) == 0 or (
+                     (value is None     and cls.Attributes.nillable) or
+                     (value is not None and value in cls.Attributes.values)
+                ))
             )
+
+
+class PushBase(object):
+    def __init__(self, callback=None, errback=None):
+        self._cb = callback
+        self._eb = errback
+
+        self.length = 0
+        self.ctx = None
+        self.app = None
+        self.response = None
+        self.gen = None
+        self._cb_finish = None
+        self._eb_finish = None
+
+    def _init(self, ctx, response, gen, _cb_finish, _eb_finish):
+        self.length = 0
+
+        self.ctx = ctx
+        self.app = ctx.app
+
+        self.response = response
+        self.gen = gen
+
+        self._cb_finish = _cb_finish
+        self._eb_finish = _eb_finish
+
+    def init(self, ctx, response, gen, _cb_finish, _eb_finish):
+        self._init(ctx, response, gen, _cb_finish, _eb_finish)
+        if self._cb is not None:
+            return self._cb(self)
+
+    def __len__(self):
+        return self.length
+
+    def append(self, inst):
+        self.gen.send(inst)
+        self.length += 1
+
+    def close(self):
+        try:
+            self.gen.throw(Break())
+        except (Break, StopIteration, GeneratorExit):
+            pass
+        self._cb_finish()
+
+
+class xml:
+    """Compound option object for xml serialization. It's meant to be passed to
+    :func:`ComplexModelBase.Attributes.store_as`.
+
+    :param root_tag: Root tag of the xml element that contains the field values.
+    :param no_ns: When true, the xml document is stripped from namespace
+        information. This is generally a stupid thing to do. Use with caution.
+    """
+
+    def __init__(self, root_tag=None, no_ns=False):
+        self.root_tag = root_tag
+        self.no_ns = no_ns
+
+
+class table:
+    """Compound option object for for storing the class instance as in row in a
+    table in a relational database. It's meant to be passed to
+    :func:`ComplexModelBase.Attributes.store_as`.
+
+    :param multi: When False, configures a one-to-many relationship where the
+        child table has a foreign key to the parent. When not ``False``,
+        configures a many-to-many relationship by creating an intermediate
+        relation table that has foreign keys to both parent and child classes
+        and generates a table name automatically. When ``True``, the table name
+        is generated automatically. Otherwise, it should be a string, as the
+        value is used as the name of the intermediate table.
+    :param left: Name of the left join column.
+    :param right: Name of the right join column.
+    :param backref: See http://docs.sqlalchemy.org/en/rel_0_9/orm/relationships.html#sqlalchemy.orm.relationship.params.backref
+    :param cascade: See http://docs.sqlalchemy.org/en/rel_0_9/orm/relationships.html#sqlalchemy.orm.relationship.params.cascade
+    :param lazy: See http://docs.sqlalchemy.org/en/rel_0_9/orm/relationships.html#sqlalchemy.orm.relationship.params.lazy
+    :param back_populates: See http://docs.sqlalchemy.org/en/rel_0_9/orm/relationships.html#sqlalchemy.orm.relationship.params.back_populates
+    """
+
+    def __init__(self, multi=False, left=None, right=None, backref=None,
+            id_backref=None, cascade=False, lazy='select', back_populates=None):
+        self.multi = multi
+        self.left = left
+        self.right = right
+        self.backref = backref
+        self.id_backref = id_backref
+        self.cascade = cascade
+        self.lazy = lazy
+        self.back_populates = back_populates
+
+
+class json:
+    """Compound option object for json serialization. It's meant to be passed to
+    :func:`ComplexModelBase.Attributes.store_as`.
+
+    Make sure you don't mix this with the json package when importing.
+    """
+
+    def __init__(self, ignore_wrappers=True, complex_as=dict):
+        if ignore_wrappers != True:
+            raise NotImplementedError("ignore_wrappers != True")
+        self.ignore_wrappers = ignore_wrappers
+        self.complex_as = complex_as
+
+
+class msgpack:
+    """Compound option object for msgpack serialization. It's meant to be passed
+    to :func:`ComplexModelBase.Attributes.store_as`.
+
+    Make sure you don't mix this with the msgpack package when importing.
+    """
+    def __init__(self):
+        pass
+
+
+def apply_pssm(val, pssm_map):
+    if val is not None:
+        val_c = pssm_map.get(val, None)
+        if val_c is None:
+            assert isinstance(val, tuple(pssm_map.values())), \
+             "'store_as' should be one of: %r or an instance of %r not %r" \
+             % (tuple(pssm_map.keys()), tuple(pssm_map.values()), val)
+
+            return val
+        return val_c()
