@@ -21,27 +21,26 @@
 """
 The ``spyne.model.primitive`` package contains types with values that fit
 in a single field.
+
+See :mod:`spyne.protocol._model` for {to,from}_string implementations.
 """
 
 
 from __future__ import absolute_import
 
 import sys
-if sys.version > '3':
-    long = int
-
 import re
 import math
-import pytz
 import uuid
 import decimal
 import datetime
 import platform
+import spyne
 
-import spyne.const.xml_ns
-
+from spyne.const import xml_ns
 from spyne.model import SimpleModel
 from spyne.util import memoize
+from spyne.model._base import apply_pssm, msgpack, xml, json
 
 string_encoding = 'utf8'
 
@@ -120,8 +119,8 @@ _duration_re = re.compile(
     )
 
 
-_ns_xs = spyne.const.xml_ns.xsd
-_ns_xsi = spyne.const.xml_ns.xsi
+_ns_xs = xml_ns.xsd
+_ns_xsi = xml_ns.xsi
 
 
 class AnyXml(SimpleModel):
@@ -140,7 +139,8 @@ class AnyXml(SimpleModel):
 
 # EXPERIMENTAL
 class AnyHtml(SimpleModel):
-    pass
+    __type_name__ = 'string'
+
 
 class AnyDict(SimpleModel):
     """A dict instance that can contain other dicts, iterables or primitive
@@ -149,6 +149,24 @@ class AnyDict(SimpleModel):
 
     __type_name__ = 'anyType'
 
+    class Attributes(SimpleModel.Attributes):
+        store_as = None
+        """Method for serializing to persistent storage. One of 'xml', 'json' or
+        'msgpack'. It makes sense to specify this only when this object is
+        child of a `ComplexModel` sublass."""
+
+    @classmethod
+    def customize(cls, **kwargs):
+        """Duplicates cls and overwrites the values in ``cls.Attributes`` with
+        ``**kwargs`` and returns the new class."""
+
+        store_as = apply_pssm(kwargs.get('store_as', None),
+                                {'json': json, 'xml': xml, 'msgpack': msgpack})
+        if store_as is not None:
+            kwargs['store_as'] = store_as
+
+        return super(AnyDict, cls).customize(**kwargs)
+
 
 class Unicode(SimpleModel):
     """The type to represent human-readable data. Its native format is `unicode`
@@ -206,8 +224,7 @@ class Unicode(SimpleModel):
     def validate_string(cls, value):
         return (     SimpleModel.validate_string(cls, value)
             and (value is None or (
-                    len(value) >= cls.Attributes.min_len
-                and len(value) <= cls.Attributes.max_len
+                cls.Attributes.min_len <= len(value) <= cls.Attributes.max_len
                 and _re_match_with_span(cls.Attributes, value)
             )))
 
@@ -293,6 +310,11 @@ class Decimal(SimpleModel):
         """A regular python string formatting string. See here:
         http://docs.python.org/library/stdtypes.html#string-formatting"""
 
+        str_format = None
+        """A regular python string formatting string used by invoking its
+        ``format()`` function. See here:
+        http://docs.python.org/2/library/string.html#format-string-syntax"""
+
         pattern = None
         """A regular expression that matches the whole field. See here for more
         info: http://www.regular-expressions.info/xml.html"""
@@ -326,9 +348,12 @@ class Decimal(SimpleModel):
 
             # + 1 for decimal separator
             # + 1 for negative sign
-            kwargs['max_str_len'] = min(cls.Attributes.total_digits +
-                                    cls.Attributes.fraction_digits + 2,
-                                                   kwargs.get('max_str_len', 0))
+            msl = kwargs.get('max_str_len', None)
+            if msl is None:
+                kwargs['max_str_len'] = (cls.Attributes.total_digits +
+                                             cls.Attributes.fraction_digits + 2)
+            else:
+                kwargs['max_str_len'] = msl
 
         retval = SimpleModel.__new__(cls,  ** kwargs)
 
@@ -341,16 +366,16 @@ class Decimal(SimpleModel):
                 and cls.Attributes.ge == Decimal.Attributes.ge
                 and cls.Attributes.lt == Decimal.Attributes.lt
                 and cls.Attributes.le == Decimal.Attributes.le
-                and cls.Attributes.total_digits == \
-                                         Decimal.Attributes.total_digits
-                and cls.Attributes.fraction_digits == \
-                                         Decimal.Attributes.fraction_digits
+                and cls.Attributes.total_digits ==
+                                             Decimal.Attributes.total_digits
+                and cls.Attributes.fraction_digits ==
+                                             Decimal.Attributes.fraction_digits
             )
 
     @staticmethod
     def validate_string(cls, value):
         return SimpleModel.validate_string(cls, value) and (
-            value is None or (len(value) <= (cls.Attributes.max_str_len))
+            value is None or (len(value) <= cls.Attributes.max_str_len)
         )
 
     @staticmethod
@@ -413,6 +438,12 @@ class Integer(Decimal):
 
     __type_name__ = 'integer'
 
+    @staticmethod
+    def validate_native(cls, value):
+        return (    Decimal.validate_native(cls, value)
+                and (value is None or int(value) == value)
+            )
+
 class UnsignedInteger(Integer):
     """The arbitrary-size unsigned integer, also known as nonNegativeInteger."""
 
@@ -421,17 +452,29 @@ class UnsignedInteger(Integer):
     @staticmethod
     def validate_native(cls, value):
         return (    Integer.validate_native(cls, value)
-                and value >= 0
+                and (value is None or value >= 0)
             )
 
 NonNegativeInteger = UnsignedInteger
 """The arbitrary-size unsigned integer, alias for UnsignedInteger."""
 
 
+class PositiveInteger(NonNegativeInteger):
+
+    """The arbitrary-size positive integer (natural number)."""
+
+    __type_name__ = 'positiveInteger'
+
+    @staticmethod
+    def validate_native(cls, value):
+        return (Integer.validate_native(cls, value)
+                and (value is None or value > 0))
+
+
 @memoize
 def TBoundedInteger(num_bits, type_name):
     _min_b = -(0x8<<(num_bits-4))     # 0x8 is 4 bits.
-    _max_b =  (0x8<<(num_bits-4)) - 1 # -1? c'est la vie ;)
+    _max_b =  (0x8<<(num_bits-4)) - 1 # -1? c'est la vie
 
     class _BoundedInteger(Integer):
         __type_name__ = type_name
@@ -444,8 +487,8 @@ def TBoundedInteger(num_bits, type_name):
         @staticmethod
         def validate_native(cls, value):
             return (
-                    _min_b <= value <= _max_b
-                and Integer.validate_native(cls, value)
+                    Integer.validate_native(cls, value)
+                and (value is None or (_min_b <= value <= _max_b))
             )
 
     return _BoundedInteger
@@ -467,8 +510,8 @@ def TBoundedUnsignedInteger(num_bits, type_name):
         @staticmethod
         def validate_native(cls, value):
             return (
-                    _min_b <= value < _max_b
-                and UnsignedInteger.validate_native(cls, value)
+                    UnsignedInteger.validate_native(cls, value)
+                and (value is None or (_min_b <= value < _max_b))
             )
 
     return _BoundedUnsignedInteger
@@ -572,18 +615,23 @@ class Time(SimpleModel):
     def validate_native(cls, value):
         return SimpleModel.validate_native(cls, value) and (
             value is None or (
-                value >  cls.Attributes.gt and
-                value >= cls.Attributes.ge and
-                value <  cls.Attributes.lt and
-                value <= cls.Attributes.le
+                    value >  cls.Attributes.gt
+                and value >= cls.Attributes.ge
+                and value <  cls.Attributes.lt
+                and value <= cls.Attributes.le
             ))
 
 
 class DateTime(SimpleModel):
     """A compact way to represent dates and times together. Supports time zones.
+    Working with timezones is a bit quirky -- Spyne works very hard to have
+    all datetimes with time zones internally and only strips them when
+    explicitly requested with ``timezone=False``\. See
+    :attr:`DateTime.Attributes.as_timezone` for more information.
 
     Native type is :class:`datetime.datetime`.
     """
+
     __type_name__ = 'dateTime'
 
     _local_re = re.compile(DATETIME_PATTERN)
@@ -594,21 +642,25 @@ class DateTime(SimpleModel):
         """Customizable attributes of the :class:`spyne.model.primitive.DateTime`
         type."""
 
-        gt = datetime.datetime(datetime.MINYEAR, 1, 1, 0, 0, 0, 0, pytz.utc) # minExclusive
-        """The datetime should be greater than this datetime."""
+        gt = datetime.datetime(datetime.MINYEAR, 1, 1, 0, 0, 0, 0, spyne.LOCAL_TZ) # minExclusive
+        """The datetime should be greater than this datetime. It must always
+        have a timezone."""
 
-        ge = datetime.datetime(datetime.MINYEAR, 1, 1, 0, 0, 0, 0, pytz.utc) # minInclusive
-        """The datetime should be greater than or equal to this datetime."""
+        ge = datetime.datetime(datetime.MINYEAR, 1, 1, 0, 0, 0, 0, spyne.LOCAL_TZ) # minInclusive
+        """The datetime should be greater than or equal to this datetime. It
+        must always have a timezone."""
 
-        lt = datetime.datetime(datetime.MAXYEAR, 12, 31, 23, 59, 59, 999999, pytz.utc) # maxExclusive
-        """The datetime should be lower than this datetime."""
+        lt = datetime.datetime(datetime.MAXYEAR, 12, 31, 23, 59, 59, 999999, spyne.LOCAL_TZ) # maxExclusive
+        """The datetime should be lower than this datetime. It must always have
+        a timezone."""
 
-        le = datetime.datetime(datetime.MAXYEAR, 12, 31, 23, 59, 59, 999999, pytz.utc) # maxInclusive
-        """The datetime should be lower than or equal to this datetime."""
+        le = datetime.datetime(datetime.MAXYEAR, 12, 31, 23, 59, 59, 999999, spyne.LOCAL_TZ) # maxInclusive
+        """The datetime should be lower than or equal to this datetime. It must
+        always have a timezone."""
 
         pattern = None
-        """A regular expression that matches the whole datetime. See here for more
-        info: http://www.regular-expressions.info/xml.html"""
+        """A regular expression that matches the whole datetime. See here for
+        more info: http://www.regular-expressions.info/xml.html"""
 
         format = None
         """DateTime format fed to the ``strftime`` function. See:
@@ -621,31 +673,28 @@ class DateTime(SimpleModel):
         string. See here for more info:
         http://docs.python.org/library/stdtypes.html#string-formatting"""
 
-        as_time_zone = None
-        """When not None, converts incoming and outgoing values to the given
-        time zone (by calling ``astimezone()``) and strips time zone information
-        from the native value (by calling ``replace(tzinfo=None)``).
+        as_timezone = None
+        """When not None, converts:
+            - Outgoing values to the given time zone (by calling
+              ``.astimezone()``).
+            - Incoming values without tzinfo to the given time zone by calling
+              ``.replace(tzinfo=<as_timezone>)`` and values with tzinfo to the
+               given timezone by calling ``.astimezone()``.
 
         Either None or a return value of pytz.timezone()
+
+        When this is None and a datetime with tzinfo=None comes in, it's
+        converted to spyne.LOCAL_TZ which defaults to ``pytz.utc``. You can use
+        `tzlocal <https://pypi.python.org/pypi/tzlocal>`_ to set it to local
+        time right after ``import spyne``.
         """
 
-    @staticmethod
-    def parse(date_match, tz=None):
-        fields = date_match.groupdict()
+        timezone = True
+        """If False, time zone info is stripped before serialization. Also makes
+        sqlalchemy schema generator emit 'timestamp without timezone'."""
 
-        year = int(fields.get('year'))
-        month =  int(fields.get('month'))
-        day = int(fields.get('day'))
-        hour = int(fields.get('hr'))
-        min = int(fields.get('min'))
-        sec = int(fields.get('sec'))
-        microsec = fields.get("sec_frac")
-        if microsec is None:
-            microsec = 0
-        else:
-            microsec = int(microsec[1:])
-
-        return datetime.datetime(year, month, day, hour, min, sec, microsec, tz)
+        serialize_as = None
+        """One of (None, 'sec', 'sec_float', 'msec', 'msec_float', 'usec')"""
 
     @staticmethod
     def is_default(cls):
@@ -659,12 +708,14 @@ class DateTime(SimpleModel):
 
     @staticmethod
     def validate_native(cls, value):
+        if isinstance(value, datetime.datetime) and value.tzinfo is None:
+            value = value.replace(tzinfo=spyne.LOCAL_TZ)
         return SimpleModel.validate_native(cls, value) and (
             value is None or (
-                value >  cls.Attributes.gt and
-                value >= cls.Attributes.ge and
-                value <  cls.Attributes.lt and
-                value <= cls.Attributes.le
+                    value >  cls.Attributes.gt
+                and value >= cls.Attributes.ge
+                and value <  cls.Attributes.lt
+                and value <= cls.Attributes.le
             ))
 
 
@@ -676,6 +727,8 @@ class Date(DateTime):
 
     __type_name__ = 'date'
 
+    _offset_re = re.compile(DATE_PATTERN + '(' + OFFSET_PATTERN + '|Z)')
+
     class Attributes(DateTime.Attributes):
         """Customizable attributes of the :class:`spyne.model.primitive.Date`
         type."""
@@ -727,19 +780,110 @@ class Boolean(SimpleModel):
     __type_name__ = 'boolean'
 
 
+def _uuid_validate_string(cls, value):
+    return (     SimpleModel.validate_string(cls, value)
+        and (value is None or (
+            cls.Attributes.min_len <= len(value) <= cls.Attributes.max_len
+            and _re_match_with_span(cls.Attributes, value)
+        )))
+
+
+def _Tuuid_validate(key):
+    from uuid import UUID
+
+    def _uvalid(cls, v):
+        try:
+            UUID(**{key:v})
+        except ValueError:
+            return False
+        return True
+    return _uvalid
+
+
+_uuid_validate = {
+    None: _uuid_validate_string,
+    'hex': _Tuuid_validate('hex'),
+    'urn': _Tuuid_validate('urn'),
+    'bytes': _Tuuid_validate('bytes'),
+    'bytes_le': _Tuuid_validate('bytes_le'),
+    'fields': _Tuuid_validate('fields'),
+    'int': _Tuuid_validate('int'),
+}
+
+
 class Uuid(Unicode(pattern=UUID_PATTERN)):
     """Unicode subclass for Universially-Unique Identifiers."""
+
     __namespace__ = 'http://spyne.io/schema'
     __type_name__ = 'uuid'
 
+    class Attributes(Unicode(pattern=UUID_PATTERN).Attributes):
+        serialize_as = None
+
+    @staticmethod
+    def validate_string(cls, value):
+        return _uuid_validate[cls.Attributes.serialize_as](cls, value)
+
+
+class NormalizedString(Unicode):
+    __type_name__ = 'normalizedString'
+    __extends__ = Unicode
+
+    class Attributes(Unicode.Attributes):
+        white_space = "replace"
+
+class Token(NormalizedString):
+    __type_name__ = 'token'
+
+    class Attributes(Unicode.Attributes):
+        white_space = "collapse"
+
+
+class Name(Token):
+    __type_name__ = 'Name'
+
+    class Attributes(Unicode.Attributes):
+        # Original: '[\i-[:]][\c-[:]]*'
+        # See: http://www.regular-expressions.info/xmlcharclass.html
+        pattern = '[[_:A-Za-z]-[:]][[-._:A-Za-z0-9]-[:]]*'
+
+
+class NCName(Name):
+    __type_name__ = 'NCName'
+
+
+
+class ID(NCName):
+    __type_name__ = 'ID'
+
+
+
+class Language(Token):
+    __type_name__ = 'language'
+
+    class Attributes(Unicode.Attributes):
+        pattern = '[a-zA-Z]{1,8}(-[a-zA-Z0-9]{1,8})*'
+
 
 class Point(Unicode):
     """A point type whose native format is a WKT string. You can use
-    :func:`shapely.wkt.loads` to get a proper point type."""
+    :func:`shapely.wkt.loads` to get a proper point type.
+
+    It's a subclass of the :class:`Unicode` type, so regular Unicode constraints
+    apply. The only additional parameter is the number of dimensions.
+
+    :param dim: Number of dimensons.
+    """
+
+    __type_name__ = None
 
     class Attributes(Unicode.Attributes):
         dim = None
 
+    @staticmethod
+    def Value(x, y, prec=15):
+        return ('POINT(%%3.%(prec)sf %%3.%(prec)sf)' % {'prec': prec}) % (x,y)
+
     def __new__(cls, dim=None, **kwargs):
         assert dim in (None,2,3)
         if dim is not None:
@@ -749,12 +893,22 @@ class Point(Unicode):
 
         retval = SimpleModel.__new__(cls, **kwargs)
         retval.__namespace__ = 'http://spyne.io/schema'
+        retval.__extends__ = Unicode
+        retval.__orig__ = Unicode
         return retval
 
 
 class Line(Unicode):
-    """A point type whose native format is a WKT string. You can use
-    :func:`shapely.wkt.loads` to get a proper point type."""
+    """A line type whose native format is a WKT string. You can use
+    :func:`shapely.wkt.loads` to get a proper line type.
+
+    It's a subclass of the :class:`Unicode` type, so regular Unicode constraints
+    apply. The only additional parameter is the number of dimensions.
+
+    :param dim: Number of dimensons.
+    """
+
+    __type_name__ = None
 
     class Attributes(Unicode.Attributes):
         dim = None
@@ -768,13 +922,23 @@ class Line(Unicode):
 
         retval = SimpleModel.__new__(cls, **kwargs)
         retval.__namespace__ = 'http://spyne.io/schema'
+        retval.__extends__ = Unicode
+        retval.__orig__ = Unicode
         return retval
 
-LineString=Line
+LineString = Line
+
 
 class Polygon(Unicode):
-    """A Polygon type whose native format is a WKT string. You can use
-    :func:`shapely.wkt.loads` to get a proper polygon type."""
+    """A polygon type whose native format is a WKT string. You can use
+    :func:`shapely.wkt.loads` to get a proper polygon type.
+
+    It's a subclass of the :class:`Unicode` type, so regular Unicode constraints
+    apply. The only additional parameter is the number of dimensions.
+
+    :param dim: Number of dimensons.
+    """
+    __type_name__ = None
 
     class Attributes(Unicode.Attributes):
         dim = None
@@ -788,12 +952,22 @@ class Polygon(Unicode):
 
         retval = SimpleModel.__new__(cls, **kwargs)
         retval.__namespace__ = 'http://spyne.io/schema'
+        retval.__extends__ = Unicode
+        retval.__orig__ = Unicode
         return retval
 
 
 class MultiPoint(Unicode):
-    """A Multipolygon type whose native format is a WKT string. You can use
-    :func:`shapely.wkt.loads` to get a proper multipolygon type."""
+    """A MultiPoint type whose native format is a WKT string. You can use
+    :func:`shapely.wkt.loads` to get a proper MultiPoint type.
+
+    It's a subclass of the :class:`Unicode` type, so regular Unicode constraints
+    apply. The only additional parameter is the number of dimensions.
+
+    :param dim: Number of dimensons.
+    """
+
+    __type_name__ = None
 
     class Attributes(Unicode.Attributes):
         dim = None
@@ -803,16 +977,26 @@ class MultiPoint(Unicode):
         if dim is not None:
             kwargs['dim'] = dim
             kwargs['pattern'] = _get_multipoint_pattern(dim)
-            kwargs['type_name'] = 'multipoint%dd' % dim
+            kwargs['type_name'] = 'multiPoint%dd' % dim
 
         retval = SimpleModel.__new__(cls, **kwargs)
         retval.__namespace__ = 'http://spyne.io/schema'
+        retval.__extends__ = Unicode
+        retval.__orig__ = Unicode
         return retval
 
 
 class MultiLine(Unicode):
-    """A Multipolygon type whose native format is a WKT string. You can use
-    :func:`shapely.wkt.loads` to get a proper multipolygon type."""
+    """A MultiLine type whose native format is a WKT string. You can use
+    :func:`shapely.wkt.loads` to get a proper MultiLine type.
+
+    It's a subclass of the :class:`Unicode` type, so regular Unicode constraints
+    apply. The only additional parameter is the number of dimensions.
+
+    :param dim: Number of dimensons.
+    """
+
+    __type_name__ = None
 
     class Attributes(Unicode.Attributes):
         dim = None
@@ -822,17 +1006,28 @@ class MultiLine(Unicode):
         if dim is not None:
             kwargs['dim'] = dim
             kwargs['pattern'] = _get_multilinestring_pattern(dim)
-            kwargs['type_name'] = 'multiline%dd' % dim
+            kwargs['type_name'] = 'multiLine%dd' % dim
 
         retval = SimpleModel.__new__(cls, **kwargs)
         retval.__namespace__ = 'http://spyne.io/schema'
+        retval.__extends__ = Unicode
+        retval.__orig__ = Unicode
         return retval
 
 MultiLineString = MultiLine
 
+
 class MultiPolygon(Unicode):
-    """A Multipolygon type whose native format is a WKT string. You can use
-    :func:`shapely.wkt.loads` to get a proper multipolygon type."""
+    """A MultiPolygon type whose native format is a WKT string. You can use
+    :func:`shapely.wkt.loads` to get a proper MultiPolygon type.
+
+    It's a subclass of the :class:`Unicode` type, so regular Unicode constraints
+    apply. The only additional parameter is the number of dimensions.
+
+    :param dim: Number of dimensons.
+    """
+
+    __type_name__ = None
 
     class Attributes(Unicode.Attributes):
         dim = None
@@ -846,13 +1041,15 @@ class MultiPolygon(Unicode):
 
         retval = SimpleModel.__new__(cls, **kwargs)
         retval.__namespace__ = 'http://spyne.io/schema'
+        retval.__extends__ = Unicode
+        retval.__orig__ = Unicode
         return retval
 
 
-# a class that is really a namespace
+# This class is DEPRECATED. Use the spyne.model.Mandatory like this:
+# >>> from spyne.model import Mandatory as M, Unicode
+# >>> MandatoryEmail = M(Unicode(pattern='[^@]+@[^@]+'))
 class Mandatory:
-    """Class that contains mandatory variants of primitives."""
-
     Unicode = Unicode(type_name="MandatoryString", min_occurs=1, nillable=False, min_len=1)
     String = String(type_name="MandatoryString", min_occurs=1, nillable=False, min_len=1)
 
@@ -869,8 +1066,8 @@ class Mandatory:
     Duration = Duration(type_name="MandatoryDuration", min_occurs=1, nillable=False)
 
     Decimal = Decimal(type_name="MandatoryDecimal", min_occurs=1, nillable=False)
-    Double = Decimal(type_name="MandatoryDouble", min_occurs=1, nillable=False)
-    Float = Double
+    Double = Double(type_name="MandatoryDouble", min_occurs=1, nillable=False)
+    Float = Float(type_name="MandatoryFloat", min_occurs=1, nillable=False)
 
     Integer = Integer(type_name="MandatoryInteger", min_occurs=1, nillable=False)
     Integer64 = Integer64(type_name="MandatoryLong", min_occurs=1, nillable=False)
@@ -918,11 +1115,11 @@ NATIVE_MAP = {
     uuid.UUID: Uuid,
 }
 
+from spyne.util import six
 
-if sys.version > '3':
+if six.PY3:
     NATIVE_MAP.update({
         str: Unicode,
-        unicode: Unicode,
         int: Integer,
     })
 
