@@ -23,7 +23,6 @@ A server that uses http as transport via wsgi. It doesn't contain any server
 logic.
 """
 
-
 import logging
 logger = logging.getLogger(__name__)
 
@@ -32,15 +31,22 @@ import threading
 import itertools
 
 try:
-    from cgi import parse_qs
-except ImportError: # Python 3
-    from urllib.parse import parse_qs
+    from urllib.parse import unquote
+except ImportError: # Python 2
+    from urlparse import unquote
 
 try:
     from werkzeug.formparser import parse_form_data
-except ImportError, e:
+except ImportError as import_error:
     def parse_form_data(*args, **kwargs):
-        raise e
+        raise import_error
+
+from spyne.util.six import string_types, BytesIO, PY3
+if PY3:
+    from http.cookies import SimpleCookie
+else:
+    from Cookie import SimpleCookie
+
 
 from spyne.application import get_fault_string_from_exception
 from spyne.auxproc import process_contexts
@@ -48,27 +54,52 @@ from spyne.error import RequestTooLongError
 from spyne.model.binary import File
 from spyne.model.fault import Fault
 from spyne.protocol.http import HttpRpc
-from spyne.protocol.http import HttpPattern
 from spyne.server.http import HttpBase
 from spyne.server.http import HttpMethodContext
 from spyne.server.http import HttpTransportContext
 from spyne.util import reconstruct_url
+from spyne.util.odict import odict
 
 from spyne.const.ansi_color import LIGHT_GREEN
 from spyne.const.ansi_color import END_COLOR
 from spyne.const.http import HTTP_200
 from spyne.const.http import HTTP_404
-from spyne.const.http import HTTP_405
 from spyne.const.http import HTTP_500
 
 
 try:
     from spyne.protocol.soap.mime import apply_mtom
-except ImportError, e:
+except ImportError as e:
     def apply_mtom(*args, **kwargs):
         raise e
 
 
+def _parse_qs(qs):
+    pairs = (s2 for s1 in qs.split('&') for s2 in s1.split(';'))
+    retval = odict()
+
+    for name_value in pairs:
+        if name_value is None or len(name_value) == 0:
+            continue
+        nv = name_value.split('=', 1)
+
+        if len(nv) != 2:
+            # Handle case of a control-name with no equal sign
+            nv.append(None)
+
+        name = unquote(nv[0].replace('+', ' '))
+
+        value = None
+        if nv[1] is not None:
+            value = unquote(nv[1].replace('+', ' '))
+
+        l = retval.get(name, None)
+        if l is None:
+            l = retval[name] = []
+        l.append(value)
+
+    return retval
+
 def _get_http_headers(req_env):
     retval = {}
 
@@ -96,8 +127,8 @@ class WsgiTransportContext(HttpTransportContext):
     """The class that is used in the transport attribute of the
     :class:`WsgiMethodContext` class."""
 
-    def __init__(self, transport, req_env, content_type):
-        HttpTransportContext.__init__(self, transport, req_env, content_type)
+    def __init__(self, parent, transport, req_env, content_type):
+        super(WsgiTransportContext, self).__init__(parent, transport, req_env, content_type)
 
         self.req_env = self.req
         """WSGI Request environment"""
@@ -105,6 +136,16 @@ class WsgiTransportContext(HttpTransportContext):
         self.req_method = req_env.get('REQUEST_METHOD', None)
         """HTTP Request verb, as a convenience to users."""
 
+    def get_cookie(self, key):
+        cookie_string = self.req_env.get('HTTP_COOKIE', None)
+        if cookie_string is None:
+            return
+
+        cookie = SimpleCookie()
+        cookie.load(cookie_string)
+
+        return cookie.get(key, None).value
+
 
 class WsgiMethodContext(HttpMethodContext):
     """The WSGI-Specific method context. WSGI-Specific information is stored in
@@ -112,103 +153,85 @@ class WsgiMethodContext(HttpMethodContext):
     """
 
     def __init__(self, transport, req_env, content_type):
-        HttpMethodContext.__init__(self, transport, req_env, content_type)
+        super(WsgiMethodContext, self).__init__(transport, req_env, content_type)
 
-        self.transport = WsgiTransportContext(transport, req_env, content_type)
+        self.transport = WsgiTransportContext(self, transport, req_env, content_type)
         """Holds the WSGI-specific information"""
 
 
 class WsgiApplication(HttpBase):
-    '''A `PEP-3333 <http://www.python.org/dev/peps/pep-3333>`_
+    """A `PEP-3333 <http://www.python.org/dev/peps/pep-3333>`_
     compliant callable class.
 
+    If you want to have a hard-coded URL in the wsdl document, this is how to do
+    it: ::
+
+        wsgi_app = WsgiApplication(...)
+        wsgi_app.doc.wsdl11.build_interface_document("http://example.com")
+
+    This is not strictly necessary -- if you don't do this, Spyne will get the
+    URL from the first request, build the wsdl on-the-fly and cache it as a
+    string in memory for later requests. However, if you want to make sure
+    you only have this url on the WSDL, this is how to do it. Note that if
+    your client takes the information in the Wsdl document seriously (not all
+    do), all requests will go to the designated url above even when you get the
+    Wsdl from another location, which can make testing a bit difficult. Use in
+    moderation.
+
     Supported events:
-    * ``wsdl``
-        Called right before the wsdl data is returned to the client.
+        * ``wsdl``
+            Called right before the wsdl data is returned to the client.
 
-    * ``wsdl_exception``
-        Called right after an exception is thrown during wsdl generation.
-        The exception object is stored in ctx.transport.wsdl_error attribute.
+        * ``wsdl_exception``
+            Called right after an exception is thrown during wsdl generation.
+            The exception object is stored in ctx.transport.wsdl_error
+            attribute.
 
-    * ``wsgi_call``
-        Called first when the incoming http request is identified as a rpc
-        request.
+        * ``wsgi_call``
+            Called first when the incoming http request is identified as a rpc
+            request.
 
-    * ``wsgi_return``
-        Called right before the output stream is returned to the WSGI handler.
+        * ``wsgi_return``
+            Called right before the output stream is returned to the WSGI
+            handler.
 
-    * ``wsgi_error``
-        Called right before returning the exception to the client.
+        * ``wsgi_exception``
+            Called right before returning the exception to the client.
 
-    * ``wsgi_close``
-        Called after the whole data has been returned to the client. It's
-        called both from success and error cases.
-    '''
+        * ``wsgi_close``
+            Called after the whole data has been returned to the client. It's
+            called both from success and error cases.
+    """
 
-    def __init__(self, app, chunked=True,
-                max_content_length=2 * 1024 * 1024,
-                block_length=8 * 1024):
-        HttpBase.__init__(self, app, chunked, max_content_length, block_length)
+    def __init__(self, app, chunked=True, max_content_length=2 * 1024 * 1024,
+                                                         block_length=8 * 1024):
+        super(WsgiApplication, self).__init__(app, chunked, max_content_length,
+                                                                   block_length)
 
-        self._allowed_http_verbs = app.in_protocol.allowed_http_verbs
-        self._verb_handlers = {
-            "GET": self.handle_rpc,
-            "POST": self.handle_rpc,
-        }
         self._mtx_build_interface_document = threading.Lock()
+
         self._wsdl = None
-
-        # Initialize HTTP Patterns
-        self._http_patterns = None
-        self._map_adapter = None
-        self._mtx_build_map_adapter = threading.Lock()
-
-        for k,v in self.app.interface.service_method_map.items():
-            # p_ stands for primary
-            p_service_class, p_method_descriptor = v[0]
-            for patt in p_method_descriptor.patterns:
-                if isinstance(patt, HttpPattern):
-                    r = patt.as_werkzeug_rule()
-
-                    # We are doing this here because we want to import Werkzeug
-                    # as late as possible.
-                    if self._http_patterns is None:
-                        from werkzeug.routing import Map
-                        self._http_patterns = Map(host_matching=True)
-
-                    self._http_patterns.add(r)
-
-    @property
-    def has_patterns(self):
-        return self._http_patterns is not None
+        if self.doc.wsdl11 is not None:
+            self._wsdl = self.doc.wsdl11.get_interface_document()
 
     def __call__(self, req_env, start_response, wsgi_url=None):
-        '''This method conforms to the WSGI spec for callable wsgi applications
+        """This method conforms to the WSGI spec for callable wsgi applications
         (PEP 333). It looks in environ['wsgi.input'] for a fully formed rpc
         message envelope, will deserialize the request parameters and call the
         method on the object returned by the get_handler() method.
-        '''
+        """
 
         url = wsgi_url
-        verb = req_env['REQUEST_METHOD'].upper()
         if url is None:
             url = reconstruct_url(req_env).split('.wsdl')[0]
 
-        if self.__is_wsdl_request(req_env):
-            return self.__handle_wsdl_request(req_env, start_response, url)
-
-        elif not (self._allowed_http_verbs is None or
-               verb in self._allowed_http_verbs or verb in self._verb_handlers):
-            start_response(HTTP_405, [
-                ('Content-Type', ''),
-                ('Allow', ', '.join(self._allowed_http_verbs)),
-            ])
-            return [HTTP_405]
+        if self.is_wsdl_request(req_env):
+            return self.handle_wsdl_request(req_env, start_response, url)
 
         else:
-            return self._verb_handlers[verb](req_env, start_response)
+            return self.handle_rpc(req_env, start_response)
 
-    def __is_wsdl_request(self, req_env):
+    def is_wsdl_request(self, req_env):
         # Get the wsdl for the service. Assume path_info matches pattern:
         # /stuff/stuff/stuff/serviceName.wsdl or
         # /stuff/stuff/stuff/serviceName/?wsdl
@@ -221,7 +244,8 @@ class WsgiApplication(HttpBase):
             )
         )
 
-    def __handle_wsdl_request(self, req_env, start_response, url):
+
+    def handle_wsdl_request(self, req_env, start_response, url):
         ctx = WsgiMethodContext(self, req_env, 'text/xml; charset=utf-8')
 
         if self.doc.wsdl11 is None:
@@ -229,6 +253,9 @@ class WsgiApplication(HttpBase):
                                   _gen_http_headers(ctx.transport.resp_headers))
             return [HTTP_404]
 
+        if self._wsdl is None:
+            self._wsdl = self.doc.wsdl11.get_interface_document()
+
         ctx.transport.wsdl = self._wsdl
 
         if ctx.transport.wsdl is None:
@@ -242,7 +269,7 @@ class WsgiApplication(HttpBase):
                     ctx.transport.wsdl = self._wsdl = \
                                         self.doc.wsdl11.get_interface_document()
 
-            except Exception, e:
+            except Exception as e:
                 logger.exception(e)
                 ctx.transport.wsdl_error = e
 
@@ -277,10 +304,10 @@ class WsgiApplication(HttpBase):
 
         if p_ctx.transport.resp_code is None:
             p_ctx.transport.resp_code = \
-                self.app.out_protocol.fault_to_http_response_code(error)
+                p_ctx.out_protocol.fault_to_http_response_code(error)
 
         self.get_out_string(p_ctx)
-        p_ctx.out_string = [''.join(p_ctx.out_string)]
+        p_ctx.out_string = [b''.join(p_ctx.out_string)]
 
         p_ctx.transport.resp_headers['Content-Length'] = \
                                                    str(len(p_ctx.out_string[0]))
@@ -291,7 +318,7 @@ class WsgiApplication(HttpBase):
 
         try:
             process_contexts(self, others, p_ctx, error=error)
-        except Exception,e:
+        except Exception as e:
             # Report but ignore any exceptions from auxiliary methods.
             logger.exception(e)
 
@@ -329,13 +356,14 @@ class WsgiApplication(HttpBase):
         try:
             self.get_out_string(p_ctx)
 
-        except Exception, e:
+        except Exception as e:
             logger.exception(e)
             p_ctx.out_error = Fault('Server', get_fault_string_from_exception(e))
-            return self.handle_error(p_ctx, others, p_ctx.out_error, start_response)
+            return self.handle_error(p_ctx, others, p_ctx.out_error,
+                                                                 start_response)
 
 
-        if isinstance(self.app.out_protocol, HttpRpc) and \
+        if isinstance(p_ctx.out_protocol, HttpRpc) and \
                                                p_ctx.out_header_doc is not None:
             p_ctx.transport.resp_headers.update(p_ctx.out_header_doc)
 
@@ -347,25 +375,25 @@ class WsgiApplication(HttpBase):
             # following test.
             out_type_info = p_ctx.descriptor.out_message._type_info
             if len(out_type_info) == 1:
-                out_object = [out_object]
+                p_ctx.out_object = [p_ctx.out_object]
 
             p_ctx.transport.resp_headers, p_ctx.out_string = apply_mtom(
                     p_ctx.transport.resp_headers, p_ctx.out_string,
                     p_ctx.descriptor.out_message._type_info.values(),
-                    out_object
+                    p_ctx.out_object,
                 )
 
         self.event_manager.fire_event('wsgi_return', p_ctx)
 
         if self.chunked:
-            # the client has not set a content-length, so we delete it as the
+            # the user has not set a content-length, so we delete it as the
             # input is just an iterable.
             if 'Content-Length' in p_ctx.transport.resp_headers:
                 del p_ctx.transport.resp_headers['Content-Length']
         else:
             p_ctx.out_string = [''.join(p_ctx.out_string)]
 
-        # if the out_string is a generator function, this hack lets the user
+        # if the out_string is a generator function, this hack makes the user
         # code run until first yield, which lets it set response headers and
         # whatnot before calling start_response. Is there a better way?
         try:
@@ -383,7 +411,7 @@ class WsgiApplication(HttpBase):
         except TypeError:
             retval_iter = iter(p_ctx.out_string)
             try:
-                first_chunk = retval_iter.next()
+                first_chunk = next(retval_iter)
             except StopIteration:
                 first_chunk = ''
 
@@ -395,7 +423,7 @@ class WsgiApplication(HttpBase):
 
         try:
             process_contexts(self, others, p_ctx, error=None)
-        except Exception, e:
+        except Exception as e:
             # Report but ignore any exceptions from auxiliary methods.
             logger.exception(e)
 
@@ -410,15 +438,14 @@ class WsgiApplication(HttpBase):
     def __reconstruct_wsgi_request(self, http_env):
         """Reconstruct http payload using information in the http header."""
 
-        # fyi, here's what the parse_header function returns:
-        # >>> import cgi; cgi.parse_header("text/xml; charset=utf-8")
-        # ('text/xml', {'charset': 'utf-8'})
         content_type = http_env.get("CONTENT_TYPE")
-        if content_type is None:
-            charset = 'utf-8'
-        else:
+        charset = None
+        if content_type is not None:
+            # fyi, here's what the parse_header function returns:
+            # >>> import cgi; cgi.parse_header("text/xml; charset=utf-8")
+            # ('text/xml', {'charset': 'utf-8'})
             content_type = cgi.parse_header(content_type)
-            charset = content_type[1].get('charset', 'utf-8')
+            charset = content_type[1].get('charset', None)
 
         return self.__wsgi_input_to_iterable(http_env), charset
 
@@ -471,41 +498,43 @@ class WsgiApplication(HttpBase):
         """
 
         params = {}
+        wsgi_env = ctx.in_document
 
         if self.has_patterns:
-            from werkzeug.exceptions import NotFound
-            if self._map_adapter is None:
-                self.generate_map_adapter(ctx)
+            # http://legacy.python.org/dev/peps/pep-0333/#url-reconstruction
+            domain = wsgi_env.get('HTTP_HOST', None)
+            if domain is None:
+                domain = wsgi_env['SERVER_NAME']
+            else:
+                domain = domain.partition(':')[0] # strip port info
 
-            try:
-                #If PATH_INFO matches a url, Set method_request_string to mrs
-                mrs, params = self._map_adapter.match(
-                                            ctx.in_document["PATH_INFO"],
-                                            ctx.in_document["REQUEST_METHOD"])
-                ctx.method_request_string = mrs
-
-            except NotFound:
-                pass
+            params = self.match_pattern(ctx,
+                    wsgi_env.get('REQUEST_METHOD', ''),
+                    wsgi_env.get('PATH_INFO', ''),
+                    domain,
+                )
 
         if ctx.method_request_string is None:
             ctx.method_request_string = '{%s}%s' % (
                                     prot.app.interface.get_tns(),
-                                    ctx.in_document['PATH_INFO'].split('/')[-1])
+                                    wsgi_env['PATH_INFO'].split('/')[-1])
 
         logger.debug("%sMethod name: %r%s" % (LIGHT_GREEN,
                                           ctx.method_request_string, END_COLOR))
 
-        ctx.in_header_doc = _get_http_headers(ctx.in_document)
-        ctx.in_body_doc = parse_qs(ctx.in_document['QUERY_STRING'])
+        ctx.in_header_doc = _get_http_headers(wsgi_env)
+        ctx.in_body_doc = _parse_qs(wsgi_env['QUERY_STRING'])
+
         for k, v in params.items():
              if k in ctx.in_body_doc:
-                 ctx.in_body_doc[k].append(v)
+                 ctx.in_body_doc[k].extend(v)
              else:
-                 ctx.in_body_doc[k] = [v]
+                 ctx.in_body_doc[k] = list(v)
 
-        if ctx.in_document['REQUEST_METHOD'].upper() in ('POST', 'PUT', 'PATCH'):
-            stream, form, files = parse_form_data(ctx.in_document,
-                                            stream_factory=prot.stream_factory)
+        verb = wsgi_env['REQUEST_METHOD'].upper()
+        if verb in ('POST', 'PUT', 'PATCH'):
+            stream, form, files = parse_form_data(wsgi_env,
+                                             stream_factory=prot.stream_factory)
 
             for k, v in form.lists():
                 val = ctx.in_body_doc.get(k, [])
@@ -528,3 +557,7 @@ class WsgiApplication(HttpBase):
                                                     path=path, handle=v.stream))
 
                 ctx.in_body_doc[k] = val
+
+            for k, v in ctx.in_body_doc.items():
+                if v == ['']:
+                    ctx.in_body_doc[k] = [None]
