@@ -17,6 +17,7 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 #
 
+import pytz
 import datetime
 import unittest
 
@@ -24,25 +25,32 @@ from pprint import pprint
 
 from lxml import etree
 
-from spyne.application import Application
-from spyne.decorator import rpc
-from spyne.service import ServiceBase
+from base64 import b64encode
+
+from spyne import Application
+from spyne import rpc
+from spyne import mrpc
+from spyne import ServiceBase
+from spyne.const import xml_ns
+from spyne.error import ResourceNotFoundError
 from spyne.interface import Interface
 from spyne.interface.wsdl import Wsdl11
 from spyne.protocol import ProtocolBase
-from spyne.const import xml_ns
+from spyne.protocol.soap import Soap11
+from spyne.server.null import NullServer
+from spyne.model import ByteArray
+from spyne.model import Array
+from spyne.model import ComplexModel
+from spyne.model import SelfReference
+from spyne.model import XmlData
+from spyne.model import XmlAttribute
+from spyne.model import Unicode
+from spyne.model import DateTime
+from spyne.model import Float
+from spyne.model import Integer
+from spyne.model import String
 
-from spyne.model.complex import Array
-from spyne.model.complex import ComplexModel
-from spyne.model.complex import SelfReference
-from spyne.model.complex import XmlAttribute
-
-from spyne.model.primitive import DateTime
-from spyne.model.primitive import Float
-from spyne.model.primitive import Integer
-from spyne.model.primitive import String
-
-from spyne.protocol.dictdoc import FlatDictDocument
+from spyne.protocol.dictdoc import SimpleDictDocument
 from spyne.protocol.xml import XmlDocument
 
 from spyne.test import FakeApp
@@ -98,7 +106,6 @@ class Level1(ComplexModel):
 Level1.resolve_namespace(Level1, __name__)
 
 class TestComplexModel(unittest.TestCase):
-
     def test_simple_class(self):
         a = Address()
         a.street = '123 happy way'
@@ -108,17 +115,17 @@ class TestComplexModel(unittest.TestCase):
         a.longitude = 88.0
 
         element = etree.Element('test')
-        XmlDocument().to_parent_element(Address, a, ns_test, element)
+        XmlDocument().to_parent(None, Address, a, element, ns_test)
         element = element[0]
         self.assertEquals(5, len(element.getchildren()))
 
-        a.since = datetime.datetime(year=2011, month=12, day=31)
+        a.since = datetime.datetime(year=2011, month=12, day=31, tzinfo=pytz.utc)
         element = etree.Element('test')
-        XmlDocument().to_parent_element(Address, a, ns_test, element)
+        XmlDocument().to_parent(None, Address, a, element, ns_test)
         element = element[0]
         self.assertEquals(6, len(element.getchildren()))
 
-        r = XmlDocument().from_element(Address, element)
+        r = XmlDocument().from_element(None, Address, element)
 
         self.assertEquals(a.street, r.street)
         self.assertEquals(a.city, r.city)
@@ -130,7 +137,7 @@ class TestComplexModel(unittest.TestCase):
     def test_nested_class(self): # FIXME: this test is incomplete
         p = Person()
         element = etree.Element('test')
-        XmlDocument().to_parent_element(Person, p, ns_test, element)
+        XmlDocument().to_parent(None, Person, p, element, ns_test)
         element = element[0]
 
         self.assertEquals(None, p.name)
@@ -140,8 +147,8 @@ class TestComplexModel(unittest.TestCase):
 
     def test_class_array(self):
         peeps = []
-        names = ['bob', 'jim', 'peabody', 'mumblesleves']
-        dob = datetime.datetime(1979, 1, 1)
+        names = ['bob', 'jim', 'peabody', 'mumblesleeves']
+        dob = datetime.datetime(1979, 1, 1, tzinfo=pytz.utc)
         for name in names:
             a = Person()
             a.name = name
@@ -154,12 +161,12 @@ class TestComplexModel(unittest.TestCase):
 
         element = etree.Element('test')
 
-        XmlDocument().to_parent_element(type, peeps, ns_test, element)
+        XmlDocument().to_parent(None, type, peeps, element, ns_test)
         element = element[0]
 
         self.assertEquals(4, len(element.getchildren()))
 
-        peeps2 = XmlDocument().from_element(type, element)
+        peeps2 = XmlDocument().from_element(None, type, element)
         for i in range(0, 4):
             self.assertEquals(peeps2[i].name, names[i])
             self.assertEquals(peeps2[i].birthdate, dob)
@@ -185,12 +192,12 @@ class TestComplexModel(unittest.TestCase):
         type = Array(Person)
         type.resolve_namespace(type, __name__)
         element = etree.Element('test')
-        XmlDocument().to_parent_element(type, peeps, ns_test, element)
+        XmlDocument().to_parent(None, type, peeps, element, ns_test)
         element = element[0]
 
         self.assertEquals(4, len(element.getchildren()))
 
-        peeps2 = XmlDocument().from_element(type, element)
+        peeps2 = XmlDocument().from_element(None, type, element)
         for peep in peeps2:
             self.assertEquals(27, peep.age)
             self.assertEquals(25, len(peep.addresses))
@@ -215,77 +222,43 @@ class TestComplexModel(unittest.TestCase):
             l.level4.append(a)
 
         element = etree.Element('test')
-        XmlDocument().to_parent_element(Level1, l, ns_test, element)
+        XmlDocument().to_parent(None, Level1, l, element, ns_test)
         element = element[0]
-        l1 = XmlDocument().from_element(Level1, element)
+        l1 = XmlDocument().from_element(None, Level1, element)
 
         self.assertEquals(l1.level2.arg1, l.level2.arg1)
         self.assertEquals(l1.level2.arg2, l.level2.arg2)
         self.assertEquals(len(l1.level4), len(l.level4))
         self.assertEquals(100, len(l.level3))
 
-    def test_customize(self):
-        class Base(ComplexModel):
-            class Attributes(ComplexModel.Attributes):
-                prop1 = 3
-                prop2 = 6
-
-        Base2 = Base.customize(prop1=4)
-
-        self.assertNotEquals(Base.Attributes.prop1, Base2.Attributes.prop1)
-        self.assertEquals(Base.Attributes.prop2, Base2.Attributes.prop2)
-
-        class Derived(Base):
-            class Attributes(Base.Attributes):
-                prop3 = 9
-                prop4 = 12
-
-        Derived2 = Derived.customize(prop1=5, prop3=12)
-
-        self.assertEquals(Base.Attributes.prop1, 3)
-        self.assertEquals(Base2.Attributes.prop1, 4)
-
-        self.assertEquals(Derived.Attributes.prop1, 3)
-        self.assertEquals(Derived2.Attributes.prop1, 5)
-
-        self.assertNotEquals(Derived.Attributes.prop3, Derived2.Attributes.prop3)
-        self.assertEquals(Derived.Attributes.prop4, Derived2.Attributes.prop4)
-
-        Derived3 = Derived.customize(prop3=12)
-        Base.prop1 = 4
-
-        # changes made to bases propagate, unless overridden
-        self.assertEquals(Derived.Attributes.prop1, Base.Attributes.prop1)
-        self.assertNotEquals(Derived2.Attributes.prop1, Base.Attributes.prop1)
-        self.assertEquals(Derived3.Attributes.prop1, Base.Attributes.prop1)
-
-
 
 class X(ComplexModel):
     __namespace__ = 'tns'
     x = Integer(nillable=True, max_occurs='unbounded')
 
+
 class Y(X):
     __namespace__ = 'tns'
     y = Integer
 
+
 class TestIncompleteInput(unittest.TestCase):
     def test_x(self):
         x = X()
         x.x = [1, 2]
         element = etree.Element('test')
-        XmlDocument().to_parent_element(X, x, 'tns', element)
+        XmlDocument().to_parent(None, X, x, element, 'tns')
         msg = element[0]
-        r = XmlDocument().from_element(X, msg)
+        r = XmlDocument().from_element(None, X, msg)
         self.assertEqual(r.x, [1, 2])
 
     def test_y_fromxml(self):
         x = X()
         x.x = [1, 2]
         element = etree.Element('test')
-        XmlDocument().to_parent_element(X, x, 'tns', element)
+        XmlDocument().to_parent(None, X, x, element, 'tns')
         msg = element[0]
-        r = XmlDocument().from_element(Y, msg)
+        r = XmlDocument().from_element(None, Y, msg)
         self.assertEqual(r.x, [1, 2])
 
     def test_y_toxml(self):
@@ -293,19 +266,18 @@ class TestIncompleteInput(unittest.TestCase):
         y.x = [1, 2]
         y.y = 38
         element = etree.Element('test')
-        XmlDocument().to_parent_element(Y, y, 'tns', element)
+        XmlDocument().to_parent(None, Y, y, element, 'tns')
         msg = element[0]
-        r = XmlDocument().from_element(Y, msg)
+        r = XmlDocument().from_element(None, Y, msg)
+
 
 class SisMsg(ComplexModel):
-    """Container with metadata for Jiva integration messages
-    carried in the MQ payload.
-    """
     data_source = String(nillable=False, min_occurs=1, max_occurs=1, max_len=50)
     direction = String(nillable=False, min_occurs=1, max_occurs=1, max_len=50)
     interface_name = String(nillable=False, min_occurs=1, max_occurs=1, max_len=50)
     crt_dt = DateTime(nillable=False)
 
+
 class EncExtractXs(ComplexModel):
     __min_occurs__ = 1
     __max_occurs__ = 1
@@ -314,16 +286,6 @@ class EncExtractXs(ComplexModel):
     hist_idn = Integer(nillable=False, min_occurs=1, max_occurs=1, max_len=18)
 
 
-class EncExtractSisMsg(SisMsg):
-    """Message indicating a Jiva episode needs to be extracted.
-
-    Desirable API: Will it work?
-    >>> msg = XmlDocument().from_element(EncExtractSisMsg, raw_xml)
-    >>> msg.body.mbr_idn
-    """
-    body = EncExtractXs
-
-
 class TestXmlAttribute(unittest.TestCase):
     def assertIsNotNone(self, obj, msg=None):
         """Stolen from Python 2.7 stdlib."""
@@ -355,6 +317,54 @@ class TestXmlAttribute(unittest.TestCase):
         self.assertEqual(attribute_def.get('name'), 'a')
         self.assertEqual(attribute_def.get('type'), CM.a.type.get_type_name_ns(interface))
 
+    def test_b64_non_attribute(self):
+        class PacketNonAttribute(ComplexModel):
+            __namespace__ = 'myns'
+            Data = ByteArray
+
+        test_string = 'yo test data'
+        b64string = b64encode(test_string)
+
+        gg = PacketNonAttribute(Data=test_string)
+
+        element = etree.Element('test')
+        Soap11().to_parent(None, PacketNonAttribute, gg, element, gg.get_namespace())
+
+        element = element[0]
+        #print etree.tostring(element, pretty_print=True)
+        data = element.find('{%s}Data' % gg.get_namespace()).text
+        self.assertEquals(data, b64string)
+        s1 = Soap11().from_element(None, PacketNonAttribute, element)
+        assert s1.Data[0] == test_string
+
+    def test_b64_attribute(self):
+        class PacketAttribute(ComplexModel):
+            __namespace__ = 'myns'
+            Data = XmlAttribute(ByteArray, use='required')
+
+        test_string = 'yo test data'
+        b64string = b64encode(test_string)
+        gg = PacketAttribute(Data=test_string)
+
+        element = etree.Element('test')
+        Soap11().to_parent(None, PacketAttribute, gg, element, gg.get_namespace())
+
+        element = element[0]
+        #print etree.tostring(element, pretty_print=True)
+        self.assertEquals(element.attrib['Data'], b64string)
+
+        s1 = Soap11().from_element(None, PacketAttribute, element)
+        assert s1.Data[0] == test_string
+
+    def test_customized_type(self):
+        class SomeClass(ComplexModel):
+            a = XmlAttribute(Integer(ge=4))
+        class SomeService(ServiceBase):
+            @rpc(SomeClass)
+            def some_call(ctx, some_class):
+                pass
+        app = Application([SomeService], 'some_tns')
+
 
 class TestSimpleTypeRestrictions(unittest.TestCase):
     def test_simple_type_info(self):
@@ -368,23 +378,25 @@ class TestSimpleTypeRestrictions(unittest.TestCase):
             s = String
 
         sti = CCM.get_simple_type_info(CCM)
+
+        pprint(sti)
         assert "i" in sti
         assert sti["i"].path == ('i',)
         assert sti["i"].type is Integer
-        assert sti["s"].parent is None
+        assert sti["s"].parent is CCM
         assert "s" in sti
         assert sti["s"].path == ('s',)
         assert sti["s"].type is String
-        assert sti["s"].parent is None
+        assert sti["s"].parent is CCM
 
-        assert "c_i" in sti
-        assert sti["c_i"].path == ('c','i')
-        assert sti["c_i"].type is Integer
-        assert sti["c_i"].parent is CCM
-        assert "c_s" in sti
-        assert sti["c_s"].path == ('c','s')
-        assert sti["c_s"].type is String
-        assert sti["c_s"].parent is CCM
+        assert "c.i" in sti
+        assert sti["c.i"].path == ('c','i')
+        assert sti["c.i"].type is Integer
+        assert sti["c.i"].parent is CM
+        assert "c.s" in sti
+        assert sti["c.s"].path == ('c','s')
+        assert sti["c.s"].type is String
+        assert sti["c.s"].parent is CM
 
     def test_simple_type_info_conflicts(self):
         class CM(ComplexModel):
@@ -396,13 +408,14 @@ class TestSimpleTypeRestrictions(unittest.TestCase):
             c_i = Float
 
         try:
-            CCM.get_simple_type_info(CCM)
+            CCM.get_simple_type_info(CCM, hier_delim='_')
         except ValueError:
             pass
         else:
             raise Exception("must fail")
 
-    def test_serialization_to_simple_dict(self):
+class TestFlatDict(unittest.TestCase):
+    def test_basic(self):
         class CM(ComplexModel):
             i = Integer
             s = String
@@ -414,15 +427,68 @@ class TestSimpleTypeRestrictions(unittest.TestCase):
 
         val = CCM(i=5, s='a', c=CM(i=7, s='b'))
 
-        d = FlatDictDocument().object_to_flat_dict(CCM, val)
+        d = SimpleDictDocument().object_to_simple_dict(CCM, val)
 
         assert d['i'] == 5
         assert d['s'] == 'a'
-        assert d['c_i'] == 7
-        assert d['c_s'] == 'b'
+        assert d['c.i'] == 7
+        assert d['c.s'] == 'b'
 
         assert len(d) == 4
 
+    def test_array_not_none(self):
+        class CM(ComplexModel):
+            i = Integer
+            s = String
+
+        class CCM(ComplexModel):
+            c = Array(CM)
+
+        val = CCM(c=[CM(i=i, s='b'*(i+1)) for i in range(2)])
+
+        d = SimpleDictDocument().object_to_simple_dict(CCM, val)
+        print(d)
+
+        assert d['c[0].i'] == 0
+        assert d['c[0].s'] == 'b'
+        assert d['c[1].i'] == 1
+        assert d['c[1].s'] == 'bb'
+
+        assert len(d) == 4
+
+    def test_array_none(self):
+        class CM(ComplexModel):
+            i = Integer
+            s = String
+
+        class CCM(ComplexModel):
+            c = Array(CM)
+
+        val = CCM()
+
+        d = SimpleDictDocument().object_to_simple_dict(CCM, val)
+        print(d)
+
+        assert len(d) == 0
+
+    def test_array_nested(self):
+        class CM(ComplexModel):
+            i = Array(Integer)
+
+        class CCM(ComplexModel):
+            c = Array(CM)
+
+        val = CCM(c=[CM(i=range(i)) for i in range(2, 4)])
+
+        d = SimpleDictDocument().object_to_simple_dict(CCM, val)
+        pprint(d)
+
+        assert d['c[0].i'] == [0,1]
+        assert d['c[1].i'] == [0,1,2]
+
+        assert len(d) == 2
+
+
 class TestSelfRefence(unittest.TestCase):
     def test_canonical_case(self):
         class TestSelfReference(ComplexModel):
@@ -477,6 +543,214 @@ class TestSelfRefence(unittest.TestCase):
     def test_array_type_name(self):
         assert Array(String, type_name='punk').__type_name__ == 'punk'
 
+    def test_ctor_kwargs(self):
+        class Category(ComplexModel):
+            id = Integer(min_occurs=1, max_occurs=1, nillable=False)
+            children = Array(Unicode)
+
+        v = Category(id=5, children=['a','b'])
+
+        assert v.id == 5
+        assert v.children == ['a', 'b']
+
+    def test_ctor_args(self):
+        class Category(ComplexModel):
+            id = XmlData(Integer(min_occurs=1, max_occurs=1, nillable=False))
+            children = Array(Unicode)
+
+        v = Category(id=5, children=['a','b'])
+
+        assert v.id == 5
+        assert v.children == ['a', 'b']
+
+        v = Category(5, children=['a','b'])
+
+        assert v.id == 5
+        assert v.children == ['a', 'b']
+
+    def test_ctor_args_2(self):
+        class Category(ComplexModel):
+            children = Array(Unicode)
+
+        class BetterCategory(Category):
+            sub_category = Unicode
+
+        v = BetterCategory(children=['a','b'], sub_category='aaa')
+
+        assert v.children == ['a', 'b']
+        assert v.sub_category == 'aaa'
+
+
+class TestMemberRpc(unittest.TestCase):
+    def test_simple(self):
+        class SomeComplexModel(ComplexModel):
+            @mrpc()
+            def put(self, ctx):
+                return "PUNK!!!"
+
+        methods = SomeComplexModel.Attributes.methods
+        print(methods)
+        assert 'put' in methods
+
+    def test_simple_customize(self):
+        class SomeComplexModel(ComplexModel):
+            @mrpc()
+            def put(self, ctx):
+                return "PUNK!!!"
+
+        methods = SomeComplexModel.customize(zart='zurt').Attributes.methods
+        print(methods)
+        assert 'put' in methods
+
+    def test_simple_with_fields(self):
+        class SomeComplexModel(ComplexModel):
+            a = Integer
+            @mrpc()
+            def put(self, ctx):
+                return "PUNK!!!"
+
+        methods = SomeComplexModel.Attributes.methods
+        print(methods)
+        assert 'put' in methods
+
+    def test_simple_with_explicit_fields(self):
+        class SomeComplexModel(ComplexModel):
+            _type_info = [('a', Integer)]
+            @mrpc()
+            def put(self, ctx):
+                return "PUNK!!!"
+
+        methods = SomeComplexModel.Attributes.methods
+        print(methods)
+        assert 'put' in methods
+
+    def test_native_call(self):
+        v = 'whatever'
+
+        class SomeComplexModel(ComplexModel):
+            @mrpc()
+            def put(self, ctx):
+                return v
+
+        assert SomeComplexModel().put(None) == v
+
+    def test_interface(self):
+        class SomeComplexModel(ComplexModel):
+            @mrpc()
+            def member_method(self, ctx):
+                pass
+
+        methods = SomeComplexModel.Attributes.methods
+        print(methods)
+        assert 'member_method' in methods
+
+        class SomeService(ServiceBase):
+            @rpc(_returns=SomeComplexModel)
+            def service_method(ctx):
+                return SomeComplexModel()
+
+        app = Application([SomeService], 'some_ns')
+
+        mmm = __name__ + '.SomeComplexModel.member_method'
+        assert mmm in app.interface.method_id_map
+
+    def test_interface_mult(self):
+        class SomeComplexModel(ComplexModel):
+            @mrpc()
+            def member_method(self, ctx):
+                pass
+
+        methods = SomeComplexModel.Attributes.methods
+        print(methods)
+        assert 'member_method' in methods
+
+        class SomeService(ServiceBase):
+            @rpc(_returns=SomeComplexModel)
+            def service_method(ctx):
+                return SomeComplexModel()
+
+            @rpc(_returns=SomeComplexModel.customize(type_name='zon'))
+            def service_method_2(ctx):
+                return SomeComplexModel()
+
+        app = Application([SomeService], 'some_ns')
+
+        mmm = __name__ + '.SomeComplexModel.member_method'
+        assert mmm in app.interface.method_id_map
+
+    def test_remote_call_error(self):
+        from spyne import mrpc
+        v = 'deger'
+
+        class SomeComplexModel(ComplexModel):
+            @mrpc(_returns=SelfReference)
+            def put(self, ctx):
+                return v
+
+        class SomeService(ServiceBase):
+            @rpc(_returns=SomeComplexModel)
+            def get(ctx):
+                return SomeComplexModel()
+
+        null = NullServer(Application([SomeService], tns='some_tns'))
+
+        try:
+            null.service.put()
+        except ResourceNotFoundError:
+            pass
+        else:
+            raise Exception("Must fail with: \"Requested resource "
+                "'{spyne.test.model.test_complex}SomeComplexModel' not found\"")
+
+    def test_signature(self):
+        class SomeComplexModel(ComplexModel):
+            @mrpc()
+            def member_method(self, ctx):
+                pass
+
+        methods = SomeComplexModel.Attributes.methods
+
+        # we use __orig__ because implicit classes are .customize(validate_freq=False)'d
+        assert methods['member_method'].in_message._type_info[0].__orig__ is SomeComplexModel
+
+    def test_self_reference(self):
+        from spyne import mrpc
+
+        class SomeComplexModel(ComplexModel):
+            @mrpc(_returns=SelfReference)
+            def method(self, ctx):
+                pass
+
+        methods = SomeComplexModel.Attributes.methods
+        assert methods['method'].out_message._type_info[0] is SomeComplexModel
+
+    def test_remote_call_success(self):
+        from spyne import mrpc
+
+        class SomeComplexModel(ComplexModel):
+            i = Integer
+            @mrpc(_returns=SelfReference)
+            def echo(self, ctx):
+                return self
+
+        class SomeService(ServiceBase):
+            @rpc(_returns=SomeComplexModel)
+            def get(ctx):
+                return SomeComplexModel()
+
+        null = NullServer(Application([SomeService], tns='some_tns'))
+
+        v = SomeComplexModel(i=5)
+        assert null.service['SomeComplexModel.echo'](v) is v
+
+    def test_order(self):
+        class CM(ComplexModel):
+            _type_info = [
+                ('a', Integer),
+                ('c', Integer(order=0))
+            ]
+
+        assert CM._type_info.keys() == ['c', 'a']
 
 if __name__ == '__main__':
     unittest.main()
